"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateServiceMethodTemplate = exports.TemplateServiceMethodSchema = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const ts = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const change_1 = require("@schematics/angular/utility/change");
const updateFile_1 = require("../utils/updateFile");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const utils_1 = require("../utils/utils");
class TemplateServiceMethodSchema {
}
exports.TemplateServiceMethodSchema = TemplateServiceMethodSchema;
function generateServiceMethodTemplate(options) {
    return () => {
        var _a;
        (_a = options.envFilePath) !== null && _a !== void 0 ? _a : (options.envFilePath = utils_1.DevEnvFile);
        let serviceMethodFilePath = (0, core_1.normalize)(`/${options.path}`);
        console.log('./' + options.transformationType);
        const templateSource = (0, schematics_1.apply)((0, schematics_1.url)('./files/' + options.transformationType), [
            (0, schematics_1.applyTemplates)(Object.assign({ capitalize: schematics_1.strings.capitalize }, options)),
            (0, schematics_1.move)(serviceMethodFilePath),
        ]);
        return (0, schematics_1.chain)([
            (0, schematics_1.mergeWith)(templateSource),
            updateServiceFile(options)
        ]);
    };
}
exports.generateServiceMethodTemplate = generateServiceMethodTemplate;
let updateServiceFile = (options) => {
    return (tree) => {
        let targetFileName = options.serviceName + ".service.ts";
        let targetFilePath = (0, core_1.normalize)(`${options.path}/${targetFileName}`);
        let targetFile = tree.read(targetFilePath);
        if (!targetFile) {
            throw new schematics_1.SchematicsException(`${targetFilePath} not found make sure the command is run in the directory of the service file`);
        }
        let sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        let serviceString = (0, utils_1.createStringObject)(options.serviceName + "Service", "Service");
        let serviceMethodString = (0, utils_1.createStringObject)(options.name, "");
        let serviceClassVar = sourceFile.statements
            .filter(ts.isClassDeclaration)
            .find((v) => {
            var _a;
            return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === serviceString.classify(false);
        });
        addServiceMethodToServiceClass(options, serviceString, serviceMethodString, serviceClassVar, targetFilePath, tree);
        addNeededImportsToServiceClass(sourceFile, serviceMethodString, options, targetFilePath, tree);
        updateConstructor(options, sourceFile, targetFilePath, tree, serviceClassVar);
        updateEnvFile(options, tree, serviceString);
        // let transformFileName = options.name+".ts"
        // let transformFilePath = normalize(`${options.path}/${transformFileName}`)
        // sourceFile = wmlCreateSourceFile(targetFilePath, tree);
        // console.log(sourceFile.getFullText())
        return tree;
    };
};
function updateConstructor(options, sourceFile, targetFilePath, tree, serviceClassVar) {
    if (options.transformationType === "scrolling") {
        sourceFile = (0, utils_1.wmlCreateSourceFile)(targetFilePath, tree);
        serviceClassVar = (0, utils_1.getServiceClassVar)(sourceFile);
        (0, utils_1.performDependencyInjection)(targetFilePath, tree, `public baseService:BaseService`, serviceClassVar);
    }
    return { sourceFile, serviceClassVar };
}
function addNeededImportsToServiceClass(sourceFile, serviceMethodString, options, targetFilePath, tree) {
    let importsToAdd = [
        {
            packageName: `./${options.name}`,
            importStatement: `import { ${serviceMethodString.capitalize(false)}UIRequestBody,${serviceMethodString.capitalize(false)}UIResponseBody,${options.name}Load, ${options.name}Success } from './${options.name}';`,
            importMembers: [
                `${serviceMethodString.capitalize(false)}UIRequestBody`,
                `${serviceMethodString.capitalize(false)}UIResponseBody`,
                `${options.name}Load`, `${options.name}Success`
            ]
        }
    ];
    if (options.transformationType === "scrolling") {
        importsToAdd.push({
            packageName: '@core/utility/scroll-utils',
            importStatement: `import { listEntitiesFail, ScrollPageParams } from '@core/utility/scroll-utils';`,
            importMembers: [
                'listEntitiesFail', 'ScrollPageParams'
            ]
        });
        importsToAdd.push({
            packageName: 'rxjs',
            importStatement: `import { retry, delay } from 'rxjs';`,
            importMembers: ["retry", "delay"]
        });
        importsToAdd.push({
            packageName: '@core/base/base.service',
            importStatement: `import { BaseService } from '@core/base/base.service';`,
            importMembers: ["BaseService"]
        });
    }
    (0, utils_1.addImportStatements)(sourceFile, targetFilePath, tree, importsToAdd);
}
function addServiceMethodToServiceClass(options, serviceString, serviceMethodString, serviceClassVar, targetFilePath, tree) {
    let serviceMethodStringObj = `${serviceMethodString.orig} = (uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody(),raw =false)=>{
    return iif(
      ()=>ENV.${serviceString.camelCase(false)}.${options.name}.automate,
      of(new ${serviceMethodString.capitalize(false)}UIResponseBody()),
      ${options.name}Load(uiBody)
        .pipe(
          concatMap((apiBody)=>{
            return this.http
              .post(ENV.${serviceString.camelCase(false)}.${options.name}.url(),apiBody)
              .pipe(raw ? tap() : map(${options.name}Success))
          })
        )
      )
    }`;
    if (options.transformationType === "scrolling") {
        serviceMethodStringObj = `${serviceMethodString.orig}ScrollPageUIRequestBody = new ${serviceMethodString.classify(false)}UIRequestBody({pageSize:10})
${serviceMethodString.orig}ScrollPageObj:ScrollPageParams<any>

${serviceMethodString.orig} = (uiBody = new ${serviceMethodString.capitalize(false)}UIRequestBody(),scrollPage?:ScrollPageParams,raw =false)=>{
  this.baseService.openOverlayLoading()
  return iif(
    ()=>ENV.${serviceString.camelCase(false)}.${options.name}.automate,
    of(new ${serviceMethodString.capitalize(false)}UIResponseBody())
    .pipe(
      delay(300)
    ),
    ${options.name}Load(uiBody)
      .pipe(
        concatMap((apiBody)=>{
          return this.http
            .post(ENV.${serviceString.camelCase(false)}.${options.name}.url(),apiBody)
            .pipe(
              retry(3),
              tap({
                next:(apiBody:any)=>{
                  this.baseService.notifyUserThatThereAreNoRecords(apiBody, uiBody);
                },
                error:()=>{
                  listEntitiesFail(uiBody)
                }
              }),
              raw ? tap() : map(${options.name}Success(scrollPage,uiBody))
            )
        })
      )
    )
    .pipe(
      this.baseService.closeOverlayLoadingViaRxjsFinalize
    )
  }`;
    }
    let occurences = serviceClassVar.members.length;
    let text = serviceClassVar.getFullText();
    let insertPos = serviceClassVar.members.pos;
    let final = serviceMethodStringObj;
    if (occurences > 0) {
        let lastMember = [
            ...serviceClassVar.members
        ].pop();
        let indentation = text.match(/\r?\n(\r?)\s*/) || [];
        final = `${indentation[0] || ' '}${serviceMethodStringObj}`;
        insertPos = lastMember.end;
    }
    let changes = [new change_1.InsertChange(targetFilePath, insertPos, final)];
    (0, updateFile_1.wmlUpdateFile)(tree, targetFilePath, changes);
}
function updateEnvFile(options, tree, serviceString) {
    var _a, _b;
    let sourceFile = (0, utils_1.wmlCreateSourceFile)(options.envFilePath, tree);
    let devEnv = (_a = sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.statements) === null || _a === void 0 ? void 0 : _a.filter(ts.isClassDeclaration).find((v) => {
        var _a;
        return ((_a = v.name) === null || _a === void 0 ? void 0 : _a.getText()) === 'DevEnv';
    });
    let methodString = `${options.name}:new WMLEndpoint({
        url:()=> this.backendDomain0 + "${options.apiRoute}",
      })`;
    let envClassProp = devEnv.members.find((prop) => {
        var _a;
        return ((_a = prop.name) === null || _a === void 0 ? void 0 : _a.getText()) === serviceString.classify(false);
    });
    if (!envClassProp) {
        let lastProperty = (_b = devEnv.members
            .filter(ts.isPropertyDeclaration).at(-1)) !== null && _b !== void 0 ? _b : (0, utils_1.wmlFindChildNode)(devEnv.members, (prop) => prop.kind === ts.SyntaxKind.OpenBraceToken, "This DevEnv class is malformed I am looking for an openBrace", false, false);
        let snippet = `\n${serviceString.camelCase(false)} = {
    ${methodString},
    }
`;
        (0, utils_1.prepareForUpdate)(snippet, options.envFilePath, tree, lastProperty.end, sourceFile.getFullText());
    }
    else {
        let serviceClassObj = (0, ast_utils_1.findNodes)(envClassProp, ts.SyntaxKind.ObjectLiteralExpression, 1)[0];
        let occurrencesCount = serviceClassObj.properties.length;
        let text = serviceClassObj.getFullText();
        let insertPos = serviceClassObj.properties.pos;
        let finalVar = methodString;
        if (occurrencesCount > 0) {
            let lastEndpointLiteral = [
                ...serviceClassObj.properties,
            ].pop();
            let indentation = (0, utils_1.getIndentation)(text);
            let endpointText = `${indentation[0] || ' '}${methodString}`;
            insertPos = lastEndpointLiteral.end;
            finalVar = `,${endpointText}`;
        }
        let changes = [new change_1.InsertChange(options.envFilePath, insertPos, finalVar)];
        (0, updateFile_1.wmlUpdateFile)(tree, options.envFilePath, changes);
    }
}
//# sourceMappingURL=index.js.map